//############################################################################
//
// LaserBoy !!!
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#include "ez_uail_gui.hpp"

//############################################################################
void spokes(ez_uail_frame_set* p_frame_set)
{
    int                frame_count,
                       spoke;
    ez_3D_float        a(0, 0, pi/6);
    ez_uail_frame_set  out;
    ez_uail_frame      frame;    

    for(frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {
        out += p_frame_set->at(frame_count);
        for(spoke = 1; spoke < 12; spoke++)
        {
            frame = p_frame_set->at(frame_count);
            frame.rotate_around_origin(a * spoke);
            out[frame_count] += frame;
        }
        out[frame_count].rotate_colors();
    }
    out.save_as_uail("spokes.ild");
    return;
}

//############################################################################
void gyro(ez_uail_frame_set* p_frame_set)
{
    int                frame_count;
    ez_3D_float        a(one_degree, one_degree, one_degree);
    ez_uail_frame      frame;
    ez_uail_frame_set  out;

    for(frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {
        frame = p_frame_set->at(frame_count);
        frame.rotate_around_origin(a * frame_count);
        out += frame;
    }
    out.save_as_uail("gyro.ild");
    return;
}

//############################################################################
void rocker(ez_uail_frame_set* p_frame_set)
{
    bool               flip_flop = true;
    int                i = 0,
                       frame_count;
    ez_3D_float        a(0, 0, one_degree);
    ez_uail_frame      frame;
    ez_uail_frame_set  out;

    for(frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {
        frame = p_frame_set->at(frame_count);
        frame.rotate_around_origin(a * i);
        
        if(!((frame_count - 45) % 90))
            flip_flop = !flip_flop;
            
        if(flip_flop)
            i++;
        else
            i--;
                    
        out += frame;
    }
    out.save_as_uail("rocker.ild");
    return;
}

//############################################################################
void echos(ez_uail_frame_set* p_frame_set)
{
    int   i,
          echo,
          delay     =  1,
          num_echos = 10;

    ez_uail_frame_set  out;
    ez_uail_frame      empty_frame(p_frame_set->front().palette_index, 1); // not really!

    for(i = 0; i < (p_frame_set->number_of_frames() + (delay * num_echos)); i++)
    {
        if(i < p_frame_set->number_of_frames())
            out += p_frame_set->at(i);
        else
            out += empty_frame;

        for(echo = 1; echo <= num_echos; echo++)
            if(    i >= (delay * echo)
                && (i - delay * echo < p_frame_set->number_of_frames())
              )
                out[i] += p_frame_set->at(i - delay * echo);
    }
    out.save_as_uail("echos.ild");
    return;
}

//############################################################################
void gears(ez_uail_frame_set* p_frame_set)
{ // requires a frame set of 4 or more frames. (Try in.ild)
    if(p_frame_set->number_of_frames() > 3)
    {
        ez_uail_frame_set  out;
        
        ez_uail_frame g20 = p_frame_set->at(1);
        ez_uail_frame g16 = p_frame_set->at(2);
        ez_uail_frame g12 = p_frame_set->at(3);

        g20.scale_around_origin(ez_3D_float(0.500, 0.500, 0.500));
        g16.scale_around_origin(ez_3D_float(0.375, 0.375, 0.375));
        g12.scale_around_origin(ez_3D_float(0.300, 0.300, 0.300));

        g20.move(ez_3D_short(-10000,      0, 0));
        g16.move(ez_3D_short( 12200, -15400, 0));
        g12.move(ez_3D_short(  5150,  19300, 0));
                    
        out += (g20 + g16 + g12);

        ez_uail_frame f20 = g20;
        ez_uail_frame f16 = g16;
        ez_uail_frame f12 = g12;

        ez_3D_float a(0, 0, -pi/60);
        ez_3D_float b(0, 0,  pi/48);
        ez_3D_float c(0, 0,  pi/36);

        for(int i = 1; i < 480; i++)
        {
            f20 = g20;
            f16 = g16;
            f12 = g12;

            f20.rotate(a * i);
            f16.rotate(b * i);
            f12.rotate(c * i);

            out += (f20 + f16 + f12);
        }
        out.save_as_uail("gears.ild");
    }
    return;
}

//############################################################################
void vortex(ez_uail_frame_set* p_frame_set)
{
    int                frame_count,
                       shrink_frame_count;
    ez_3D_float        s(0.80, 0.80, 0.80);
    ez_uail_frame      frame;
    ez_uail_frame_set  out;

    for(frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {        
        frame = p_frame_set->at(frame_count);
        for(shrink_frame_count = 0; shrink_frame_count < 19; shrink_frame_count++)
        {        
            out += frame;
            frame.scale_around_origin(s);
        }
    }
    out.save_as_uail("vortex.ild");
}

//############################################################################
void fader(ez_uail_frame_set* p_frame_set)
{
    float              frame_count,
                       total = p_frame_set->number_of_frames();
    ez_uail_frame      frame;
    ez_uail_frame_set  out;

    for(frame_count = 0; frame_count < total; frame_count++)
    {        
        frame = p_frame_set->at((u_int)frame_count);
        frame.shade((u_char)(frame_count / total * 255.0));
        out += frame;
    }
    out.save_as_uail("fader.ild");
}

//############################################################################
void fader_z(ez_uail_frame_set* p_frame_set)
{
    int                i,
                       shade;
    float              frame_count,
                       total = p_frame_set->number_of_frames();
    ez_uail_frame      frame;
    ez_uail_frame_set  out;
    
    for(frame_count = 0; frame_count < total; frame_count++)
    {        
        frame = p_frame_set->at((u_int)frame_count);
        frame.promote_to_true_color();
        for(i = 1; i < frame.number_of_points(); i++)
        {
            shade = (int)(255 - (frame.at(i).z + 22768.0) / 45536.0 * 255.0);
            if(shade > 254)    shade = 255;
            if(shade < 1  )    shade = 0  ;
            (frame.at(i).r - shade > 0) ? (frame.at(i).r -= shade) : (frame.at(i).r = 0);
            (frame.at(i).g - shade > 0) ? (frame.at(i).g -= shade) : (frame.at(i).g = 0);
            (frame.at(i).b - shade > 0) ? (frame.at(i).b -= shade) : (frame.at(i).b = 0);
        }
        out += frame;
    }
    out.save_as_uail("fader_z.ild");
}

//############################################################################
void depth(ez_uail_frame_set* p_frame_set)
{
    int                i,
                       shade;

    float              frame_count,
                       scale,
                       total = p_frame_set->number_of_frames();


    ez_uail_frame      frame;
    ez_uail_frame_set  out;
    
    for(frame_count = 0; frame_count < total; frame_count++)
    {        
        frame = p_frame_set->at((u_int)frame_count);
        for(i = 0; i < frame.number_of_points(); i++)
        {
            scale = 0.8 + (frame.at(i).z + 32768.0) / 131077.0;
            frame.at(i).x = (short)(frame.at(i).x * scale);
            frame.at(i).y = (short)(frame.at(i).y * scale);
            frame.at(i).z = (short)(frame.at(i).z * scale);
        }
        out += frame;
    }
    out.save_as_uail("depth.ild");
}

//############################################################################
void drain(ez_uail_frame_set* p_frame_set)
{
    ez_uail_frame_set  out;
    ez_uail_frame      frame;
    int                frame_count,
                       shrink_frame_count;
    ez_3D_float        s(0.85, 0.85, 0.85),
                       a(0.0,  0.0,  pi/15.0);
    for(frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {        
        frame = p_frame_set->at(frame_count);
        for(shrink_frame_count = 0; shrink_frame_count < 29; shrink_frame_count++)
        {        
            out += frame;
            frame.scale_around_origin(s);
            frame.rotate_around_origin(a);
        }
    }
    out.save_as_uail("drain.ild");
}

//############################################################################
void diminish(ez_uail_frame_set* p_frame_set)
{
    ez_uail_frame_set  out;
    ez_uail_frame      frame;
    int                frame_count,
                       shrink_frame_count;
    ez_3D_float        s(0.75, 0.75, 0.75);
    for(frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {        
        frame = p_frame_set->at(frame_count);
        for(shrink_frame_count = 0; shrink_frame_count < 19; shrink_frame_count++)
        {        
            out += frame;
            frame.scale(s);
        }
    }
    out.save_as_uail("diminish.ild");
}

//############################################################################
void tess(ez_uail_frame_set* p_frame_set)
{
    ez_uail_frame_set  out;
    ez_uail_frame      frame;
    int                frame_count,
                       shrink_frame_count;
    ez_3D_float        s(0.9, 0.9, 0.9);
    for(frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {        
        frame = p_frame_set->at(frame_count);
        for(shrink_frame_count = 0; shrink_frame_count < 19; shrink_frame_count++)
        {        
            out += frame;
            frame.scale_around_origin(s);
        }
    }
    out.save_as_uail("diminish.ild");
}

//############################################################################










//############################################################################
void ez_uail_app::load_frame_set_effects()
{
    frame_set_effects.reserve(7);
    frame_set_effect_names.reserve(7);
    //------------------------------------------------------------------------
    frame_set_effects.push_back(spokes);
    frame_set_effect_names.push_back(string("spokes"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(gyro);
    frame_set_effect_names.push_back(string("gyro"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(rocker);
    frame_set_effect_names.push_back(string("rocker"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(echos);
    frame_set_effect_names.push_back(string("echos"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(gears);
    frame_set_effect_names.push_back(string("gears"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(vortex);
    frame_set_effect_names.push_back(string("vortex"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(fader);
    frame_set_effect_names.push_back(string("fader"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(fader_z);
    frame_set_effect_names.push_back(string("fader_z"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(depth);
    frame_set_effect_names.push_back(string("depth"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(drain);
    frame_set_effect_names.push_back(string("drain"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(diminish);
    frame_set_effect_names.push_back(string("diminish"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(tess);
    frame_set_effect_names.push_back(string("tess"));
    //------------------------------------------------------------------------
    return;
}

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
